// FreqMeasure.c
// Runs on LM3S811
// Use Timer0B in edge count mode to count positive edges within a
// period set by Timer0A in periodic mode.  In Timer0A periodic
// interrupts, calculate frequency.
// Daniel Valvano
// June 27, 2011

/* This example accompanies the book
   "Embedded Systems: Real Time Interfacing to the Arm Cortex M3",
   ISBN: 978-1463590154, Jonathan Valvano, copyright (c) 2011

 Copyright 2011 by Jonathan W. Valvano, valvano@mail.utexas.edu
    You may use, edit, run or distribute this file
    as long as the above copyright notice remains
 THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 VALVANO SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 For more information about my classes, my research, and my books, see
 http://users.ece.utexas.edu/~valvano/
 */

// external signal connected to PC5 (CCP1) (trigger on rising edge)
// debugging heartbeat connected to PD4 (high while in interrupt)

#define NVIC_EN0_INT19          0x00080000  // Interrupt 19 enable
#define NVIC_EN0_R              (*((volatile unsigned long *)0xE000E100))  // IRQ 0 to 31 Set Enable Register
#define NVIC_PRI4_R             (*((volatile unsigned long *)0xE000E410))  // IRQ 16 to 19 Priority Register
#define TIMER0_CFG_R            (*((volatile unsigned long *)0x40030000))
#define TIMER0_TAMR_R           (*((volatile unsigned long *)0x40030004))
#define TIMER0_TBMR_R           (*((volatile unsigned long *)0x40030008))
#define TIMER0_CTL_R            (*((volatile unsigned long *)0x4003000C))
#define TIMER0_IMR_R            (*((volatile unsigned long *)0x40030018))
#define TIMER0_ICR_R            (*((volatile unsigned long *)0x40030024))
#define TIMER0_TAILR_R          (*((volatile unsigned long *)0x40030028))
#define TIMER0_TBILR_R          (*((volatile unsigned long *)0x4003002C))
#define TIMER0_TBMATCHR_R       (*((volatile unsigned long *)0x40030034))
#define TIMER0_TAR_R            (*((volatile unsigned long *)0x40030048))
#define TIMER0_TBR_R            (*((volatile unsigned long *)0x4003004C))
#define TIMER_CFG_16_BIT        0x00000004  // 16-bit timer configuration,
                                            // function is controlled by bits
                                            // 1:0 of GPTMTAMR and GPTMTBMR
#define TIMER_TAMR_TAMR_PERIOD  0x00000002  // Periodic Timer mode
#define TIMER_TBMR_TBMR_CAP     0x00000003  // Capture mode
#define TIMER_CTL_TAEN          0x00000001  // GPTM TimerA Enable
#define TIMER_CTL_TBEN          0x00000100  // GPTM TimerB Enable
#define TIMER_CTL_TBEVENT_M     0x00000C00  // GPTM TimerB Event Mask
#define TIMER_CTL_TBEVENT_POS   0x00000000  // Positive edge
#define TIMER_IMR_TATOIM        0x00000001  // GPTM TimerA Time-Out Interrupt
                                            // Mask
#define TIMER_ICR_TATOCINT      0x00000001  // GPTM TimerA Time-Out Raw
                                            // Interrupt
#define TIMER_TAILR_TAILRL_M    0x0000FFFF  // GPTM TimerA Interval Load
                                            // Register Low
#define TIMER_TBILR_TBILRL_M    0x0000FFFF  // GPTM TimerB Interval Load
                                            // Register
#define GPIO_PORTC_AFSEL_R      (*((volatile unsigned long *)0x40006420))
#define GPIO_PORTC_DEN_R        (*((volatile unsigned long *)0x4000651C))
#define GPIO_PORTD_DIR_R        (*((volatile unsigned long *)0x40007400))
#define GPIO_PORTD_DEN_R        (*((volatile unsigned long *)0x4000751C))
#define GPIO_PORTD4             (*((volatile unsigned long *)0x40007040))
#define SYSCTL_RCGC1_R          (*((volatile unsigned long *)0x400FE104))
#define SYSCTL_RCGC2_R          (*((volatile unsigned long *)0x400FE108))
#define SYSCTL_RCGC1_TIMER0     0x00010000  // timer 0 Clock Gating Control
#define SYSCTL_RCGC2_GPIOD      0x00000008  // port D Clock Gating Control
#define SYSCTL_RCGC2_GPIOC      0x00000004  // port C Clock Gating Control

void DisableInterrupts(void); // Disable interrupts
void EnableInterrupts(void);  // Enable interrupts
long StartCritical (void);    // previous I bit, disable interrupts
void EndCritical(long sr);    // restore I bit to previous value
void WaitForInterrupt(void);  // low power mode

unsigned long Freq;   /* Frequency with units of 100 Hz */
unsigned char Done;  /* Set each measurement, every 10 ms */
void FreqMeasure_Init(void){
  // **** general initialization ****
  SYSCTL_RCGC1_R |= SYSCTL_RCGC1_TIMER0;// activate timer0
                                        // activate port C and port D
  SYSCTL_RCGC2_R |= SYSCTL_RCGC2_GPIOC+SYSCTL_RCGC2_GPIOD;
  Freq = 0;                        // allow time to finish activating
  Done = 0;
  GPIO_PORTC_DEN_R |= 0x20;        // enable digital I/O on PC5
  GPIO_PORTC_AFSEL_R |= 0x20;      // enable alt funct on PC5
  GPIO_PORTD4 = 0x00;              // turn off PD4
  GPIO_PORTD_DIR_R |= 0x10;        // make PD4 out (PD4 debug heartbeat)
  GPIO_PORTD_DEN_R |= 0x10;        // enable digital I/O on PD4
                                   // disable timer0A and timer0B during setup
  TIMER0_CTL_R &= ~(TIMER_CTL_TAEN|TIMER_CTL_TBEN);
  TIMER0_CFG_R = TIMER_CFG_16_BIT; // configure for 16-bit timer mode
  // **** timer0A initialization ****
                                   // configure for periodic mode
  TIMER0_TAMR_R = TIMER_TAMR_TAMR_PERIOD;
  TIMER0_TAILR_R = 60000;          // start value for 100 Hz interrupts
  TIMER0_IMR_R |= TIMER_IMR_TATOIM;// enable timeout (rollover) interrupt
  TIMER0_ICR_R = TIMER_ICR_TATOCINT;// clear timer0A timeout flag
  // **** timer0B initialization ****
                                   // configure for capture mode
  TIMER0_TBMR_R = TIMER_TBMR_TBMR_CAP;
                                   // clear trigger event field
  TIMER0_CTL_R &= ~TIMER_CTL_TBEVENT_M;
                                   // configure for rising edge event
  TIMER0_CTL_R += TIMER_CTL_TBEVENT_POS;
  TIMER0_TBILR_R = TIMER_TBILR_TBILRL_M;// maximum start value
  TIMER0_TBMATCHR_R = 0;           // count down to zero
  TIMER0_IMR_R &= ~0x700;          // disable all interrupts for timer0B
                                   // enable timer0A 16-b, periodic, interrupts
                                   // enable timer0B 16-b, +edge count, no interrupts
  TIMER0_CTL_R |= (TIMER_CTL_TAEN|TIMER_CTL_TBEN);// enable timers together
  // **** interrupt initialization ****
                                   // Timer0A=priority 2
  NVIC_PRI4_R = (NVIC_PRI4_R&0x00FFFFFF)|0x40000000; // top 3 bits
  NVIC_EN0_R |= NVIC_EN0_INT19;    // enable interrupt 19 in NVIC
  EnableInterrupts();
}
void Timer0A_Handler(void){
  GPIO_PORTD4 = 0x10;              // heartbeat
  TIMER0_ICR_R = TIMER_ICR_TATOCINT;// acknowledge timer0A timeout
  Freq = 100*(TIMER0_TBILR_R-TIMER0_TBR_R);// f = (pulses)/(fixed time)
  Done = 0xFF;
  // timer0B needs to be re-initialized
  TIMER0_CTL_R &= ~TIMER_CTL_TBEN; // disable timer0B during setup
                                   // configure for capture mode
  TIMER0_TBMR_R = TIMER_TBMR_TBMR_CAP;
                                   // clear trigger event field
  TIMER0_CTL_R &= ~TIMER_CTL_TBEVENT_M;
                                   // configure for rising edge event
  TIMER0_CTL_R += TIMER_CTL_TBEVENT_POS;
  TIMER0_TBILR_R = TIMER_TBILR_TBILRL_M;// maximum start value
  TIMER0_TBMATCHR_R = 0;           // count down to zero
  TIMER0_IMR_R &= ~0x700;          // disable all interrupts for timer0B
  TIMER0_CTL_R |= TIMER_CTL_TBEN;  // re-enable timer0B
  GPIO_PORTD4 = 0x00;
}

//debug code
int main(void){
  FreqMeasure_Init();              // initialize timer0
  while(1){
    WaitForInterrupt();
  }
}
